﻿// Author - Anshu Dutta
// email - anshu.dutta@gmail.com
using System;
using System.Threading;
using System.ComponentModel;
using System.Windows.Forms;

namespace Demo.Threading
{
    public partial class frmThreads : Form
    {
        #region Private Members

        private delegate void delUpdateUI(bool value, string text);

        private bool cancelled;
        private int upCount;
        private delUpdateUI dUpdateUI;
        private Thread doWork;

        #endregion

        public frmThreads()
        {
            InitializeComponent();
            Initialize();
        }

        #region Initialize
        private void Initialize()
        {
            this.btnCancel.Enabled = false;
            upCount = Convert.ToInt32(this.nudUpCount.Value);
            dUpdateUI = new delUpdateUI(UpdateUI);

            this.bgCount.DoWork +=
                new DoWorkEventHandler(bgCount_DoWork);
            this.bgCount.ProgressChanged +=
                new ProgressChangedEventHandler(bgCount_ProgressChanged);
            this.bgCount.RunWorkerCompleted +=
                new RunWorkerCompletedEventHandler(bgCount_RunWorkerCompleted);
        }
        #endregion
         
        #region Sync Calls

        private void btnSync_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            this.txtContents.Text = Utility.LoadData(upCount);
            this.Cursor = Cursors.Default;
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            this.txtContents.Text = string.Empty;
        }   
        #endregion

        #region Async Calls

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAsync_Click(object sender, EventArgs e)
        {
            this.btnCancel.Enabled = true;
            this.btnAsync.Enabled = false;

            if (this.rbDel.Checked==true)
            {                
                this.txtContents.Text = string.Empty;
                
                Utility.dLoadData = new Utility.delLoadData(Utility.LoadData);
                Utility.dLoadData.BeginInvoke(upCount, CallBack, null);
                
            }
            else if(this.rbThread.Checked==true)
            {                
                doWork = new Thread(new ThreadStart(this.LoadData));
                doWork.Start();
            }
            else
            {
                this.bgCount.RunWorkerAsync();
            }            

        }

        
        /// <summary>
        /// Callback for thread
        /// </summary>
        /// <param name="asyncResult"></param>
        private void CallBack(IAsyncResult asyncResult)
        {
            string result= string.Empty;

            if (this.cancelled)
                result = "Operation Cancelled";
            else
                result = Utility.dLoadData.EndInvoke(asyncResult);
            
            object[] args = { this.cancelled, result };
            this.BeginInvoke(dUpdateUI, args);                
                        
        }              

        /// <summary>
        /// Method for updating the UI
        /// </summary>
        /// <param name="cancelled"></param>
        /// <param name="text"></param>
        private void UpdateUI(bool cancelled, string text)
        {
            this.btnAsync.Enabled = true;
            this.btnCancel.Enabled = false;
            this.txtContents.Text = text;                                  
        }

        /// <summary>
        /// Cancel button click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.cancelled = true;
            if (rbDel.Checked==true)
            {
                
            }
            else if (rbThread.Checked == true)
            {
                doWork.Abort();
                string result = "Operation Cancelled";
                object[] args = { this.cancelled, result };
                this.BeginInvoke(dUpdateUI, args);
            }
            else
            {
                this.bgCount.CancelAsync();
            }
        }

        /// <summary>
        /// Wrapper for Utili LoadData() method
        /// </summary>
        private void LoadData()
        {
            string result = Utility.LoadData(upCount);
            object[] args = { this.cancelled, result };
            this.BeginInvoke(dUpdateUI, args);
        }

        #endregion              
        
        #region Background Thread Worker

        /// <summary>
        /// Method called for executing operation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void bgCount_DoWork(object sender, DoWorkEventArgs e)
        {
            string result = string.Empty;
            if (this.bgCount.CancellationPending)
            {
                e.Cancel = true;
                e.Result = "Operation Cancelled";
            }
            else
            {
                for (int i = 0; i < this.upCount; i++)
                {
                    result += i.ToString();
                    double progress = ((double)i / (double) this.upCount);
                    this.bgCount.ReportProgress(Convert.ToInt16(progress*100));
                }
                e.Result = result;
            }

        }

        /// <summary>
        /// Method called for updating the progress bar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void bgCount_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (this.bgCount.CancellationPending)
                this.txtContents.Text = "Cancelling....";
            else
                this.progressBar.Value = e.ProgressPercentage;

        }

        /// <summary>
        /// Method called when background worker has completed operation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void bgCount_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.btnAsync.Enabled = true;
            this.btnCancel.Enabled = false;
            this.txtContents.Text = e.Result.ToString();
        }

        #endregion
        
    }
}
